/*--------------------------------------------------------------------------*/
/* ALBERTA_UTIL:  tools for messages, memory allocation, parameters, etc.   */
/*                                                                          */
/* file: alberta_util.h                                                     */
/*                                                                          */
/* description: public header file of the ALBERTA_UTIL package              */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/*                                                                          */
/*  authors:   Alfred Schmidt                                               */
/*             Zentrum fuer Technomathematik                                */
/*             Fachbereich 3 Mathematik/Informatik                          */
/*             Universitaet Bremen                                          */
/*             Bibliothekstr. 2                                             */
/*             D-28359 Bremen, Germany                                      */
/*                                                                          */
/*             Kunibert G. Siebert                                          */
/*             Institut fuer Mathematik                                     */
/*             Universitaet Augsburg                                        */
/*             Universitaetsstr. 14                                         */
/*             D-86159 Augsburg, Germany                                    */
/*                                                                          */
/*  http://www.alberta-fem.de/                                              */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/*  (c) by A. Schmidt and K.G. Siebert (1996-2004)                          */
/*                                                                          */
/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License as published by */
/*     the Free Software Foundation; either version 2 of the License, or    */
/*     any later version.                                                   */
/*                                                                          */
/*     This program is distributed in the hope that it will be useful,      */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*     GNU General Public License for more details.                         */
/*--------------------------------------------------------------------------*/

#ifndef _ALBERTA_UTIL_H_
#define _ALBERTA_UTIL_H_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef true
#define true          1
#define false         0
#endif

#ifndef nil
#define nil           NULL
#endif

#ifndef MAX
#define MAX(a, b)     ((a) > (b) ? (a) : (b))
#define MIN(a, b)     ((a) < (b) ? (a) : (b))
#endif
#ifndef ABS
#define ABS(a)        ((a) >= 0 ? (a) : -(a))
#endif

#ifndef SQR
#define SQR(a)        ((a)*(a))
#endif

#ifndef M_E
#define M_E		2.7182818284590452354
#endif
#ifndef M_LOG2E
#define M_LOG2E		1.4426950408889634074
#endif
#ifndef M_LOG10E
#define M_LOG10E	0.43429448190325182765
#endif
#ifndef M_LN2
#define M_LN2		0.69314718055994530942
#endif
#ifndef M_LN10
#define M_LN10		2.30258509299404568402
#endif
#ifndef M_PI
#define M_PI		3.14159265358979323846
#endif
#ifndef M_PI_2
#define M_PI_2		1.57079632679489661923
#endif
#ifndef M_PI_4
#define M_PI_4		0.78539816339744830962
#endif
#ifndef M_1_PI
#define M_1_PI		0.31830988618379067154
#endif
#ifndef M_2_PI
#define M_2_PI		0.63661977236758134308
#endif
#ifndef M_2_SQRTPI
#define M_2_SQRTPI	1.12837916709551257390
#endif
#ifndef M_SQRT2
#define M_SQRT2		1.41421356237309504880
#endif
#ifndef M_SQRT1_2
#define M_SQRT1_2	0.70710678118654752440
#endif

/*--------------------------------------------------------------------------*/
/*  Definition of basic data types      				    */
/*--------------------------------------------------------------------------*/

#ifndef REAL_DEFINED
#define REAL_DEFINED

#define USE_DOUBLE_AS_REAL  1

#if USE_DOUBLE_AS_REAL

typedef double              REAL;

#define REAL_MANT_DIG     DBL_MANT_DIG     
#define REAL_EPSILON      DBL_EPSILON      
#define REAL_DIG          DBL_DIG          
#define REAL_MIN_EXP      DBL_MIN_EXP      
#define REAL_MIN          DBL_MIN          
#define REAL_MIN_10_EXP   DBL_MIN_10_EXP   
#define REAL_MAX_EXP      DBL_MAX_EXP      
#define REAL_MAX          DBL_MAX          
#define REAL_MAX_10_EXP   DBL_MAX_10_EXP   

#else  /*  USE_DOUBLE_AS_REAL  */

typedef float             REAL;

#define REAL_MANT_DIG     FLT_MANT_DIG     
#define REAL_EPSILON      FLT_EPSILON      
#define REAL_DIG          FLT_DIG          
#define REAL_MIN_EXP      FLT_MIN_EXP      
#define REAL_MIN          FLT_MIN          
#define REAL_MIN_10_EXP   FLT_MIN_10_EXP   
#define REAL_MAX_EXP      FLT_MAX_EXP      
#define REAL_MAX          FLT_MAX          
#define REAL_MAX_10_EXP   FLT_MAX_10_EXP   

#endif  /*  USE_DOUBLE_AS_REAL  */

#ifndef LARGE
#define LARGE         REAL_MAX
#endif

#endif  /*  REAL_DEFINED  */

#ifndef U_CHAR_DEFINED
#define U_CHAR_DEFINED

typedef unsigned char       U_CHAR;
typedef signed char         S_CHAR;
typedef unsigned long       FLAGS;
#endif

#ifndef WORKSPACE_DEFINED
#define WORKSPACE_DEFINED
typedef struct workspace    WORKSPACE;

struct workspace
{
  size_t  size;
  void    *work;
};
#endif

#ifndef VOID_LIST_ELEMENT_DEFINED
#define VOID_LIST_ELEMENT_DEFINED
typedef struct void_list_element VOID_LIST_ELEMENT;

struct void_list_element
{
  void               *data;
  VOID_LIST_ELEMENT  *next;
};
VOID_LIST_ELEMENT *get_void_list_element(void);
void free_void_list_element(VOID_LIST_ELEMENT *);
#endif

/*--------------------------------------------------------------------------*/
/*  functions and macros for messages	                         	    */
/*--------------------------------------------------------------------------*/

#ifdef __GNUC__
# define ALBERTA_UNUSED(proto) proto __attribute__((unused))
# define ALBERTA_DEFUNUSED(proto) ALBERTA_UNUSED(proto); proto
# define ALBERTA_NORETURN(proto) proto __attribute__((noreturn))
# define ALBERTA_DEFNORETURN(proto) ALBERTA_NORETURN(proto); proto
#else
# define ALBERTA_UNUSED(proto) proto
# define ALBERTA_DEFUNUSED(proto) proto
# define ALBERTA_NORETURN(proto) proto
# define ALBERTA_DEFNORETURN(proto) proto
#endif

void print_msg(const char *format, ...);
void print_error_msg(const char *format, ...);
ALBERTA_NORETURN(void print_error_msg_exit(const char *format, ...));
void print_error_funcname(const char *name, const char *file, int line);
void print_warn_msg(const char *format, ...);
void print_warn_funcname(const char *name, const char *file, int line);
void print_funcname(const char *name);
void print_int_vec(const char *s, const int *vec, int no);
void print_real_vec(const char *s, const REAL *vec, int no);

void change_msg_out(FILE *fp);
void open_msg_file(const char *filename, const char *type);
void change_error_out(FILE *fp);
void open_error_file(const char *filename, const char *type);
const char *generate_filename(const char *, const char *, int);

#define FUNCNAME(nn)    static const char *funcName = nn

#define ERROR           print_error_funcname(funcName, __FILE__, __LINE__),\
                        print_error_msg
#define ERROR_EXIT      print_error_funcname(funcName, __FILE__, __LINE__),\
                        print_error_msg_exit
#define WARNING         print_warn_funcname(funcName, __FILE__, __LINE__),\
                        print_warn_msg

#define TEST(test)      if ((test));else ERROR
#define TEST_EXIT(test) if ((test));else ERROR_EXIT

#define MSG             print_funcname(funcName), print_msg

#define INFO(info,noinfo)\
 if (!(msg_info&&(MIN(msg_info,(info))>=(noinfo))));else MSG

#define PRINT_INFO(info,noinfo)\
 if (!(msg_info&&(MIN(msg_info,(info))>=(noinfo))));else print_msg

#define PRINT_INT_VEC   print_funcname(funcName), print_int_vec
#define PRINT_REAL_VEC  print_funcname(funcName), print_real_vec

extern void alberta_wait(const char *, int);
#define WAIT            alberta_wait(funcName, msg_wait)
#define WAIT_REALLY     alberta_wait(funcName, 1)

/*--------------------------------------------------------------------------*/
/*  Definition of extern variables      				    */
extern int           msg_wait;                       /* declared in msg.c   */
extern const char    *funcName;                      /* declared in msg.c   */
extern int           msg_info;                       /* declared in msg.c   */

/*--------------------------------------------------------------------------*/
/*  Macros for memory allocation					    */
/*--------------------------------------------------------------------------*/

void *alberta_alloc(size_t, const char *, const char *,int);
void *alberta_realloc(void *, size_t, size_t, const char *, const char *, int);
void *alberta_calloc(size_t, size_t, const char *, const char *,int);
void alberta_free(void *, size_t);
void print_mem_use(void);

void **alberta_matrix(size_t, size_t, size_t, const char *, const char *, int);
void free_alberta_matrix(void **, size_t, size_t, size_t);

WORKSPACE *get_workspace(size_t, const char *, const char *, int);
#define GET_WORKSPACE(s) get_workspace(s,funcName,__FILE__,__LINE__)

WORKSPACE  *realloc_workspace(WORKSPACE *ws, size_t new_size, 
			      const char *fct, const char *file, int line);
#define REALLOC_WORKSPACE(ws,size)\
 (realloc_workspace(ws,size,funcName,__FILE__,__LINE__))

void clear_workspace(WORKSPACE *ws);
#define CLEAR_WORKSPACE(ws) clear_workspace(ws)
void free_workspace(WORKSPACE *ws);
#define FREE_WORKSPACE(ws) free_workspace(ws)

#define MEM_ALLOC(n,type)\
   ((type*)alberta_alloc((n)*sizeof(type),funcName,__FILE__,__LINE__))
#define MEM_REALLOC(ptr,old_n,new_n,type)\
   ((type*)alberta_realloc((void *)ptr,(old_n)*sizeof(type),\
    (new_n)*sizeof(type),funcName,__FILE__,__LINE__))
#define MEM_CALLOC(n,type)\
   ((type*)alberta_calloc((n), sizeof(type),funcName,__FILE__,__LINE__))
#define MEM_FREE(ptr,n,type) alberta_free((void *)ptr,(n)*sizeof(type))

#define MAT_ALLOC(nr,nc,type)\
  ((type**)alberta_matrix((nr),(nc),sizeof(type),funcName,__FILE__, __LINE__))
#define MAT_FREE(ptr,nr,nc,type)\
    free_alberta_matrix((void **)ptr,(nr),(nc),sizeof(type))

#define NAME(struct)\
   ((struct) ? ((struct)->name ? (struct)->name : #struct "->name unknown") :\
   #struct " pointer to nil")

#define GET_STRUCT(struct, from)\
   TEST_EXIT((from)  && (struct = (from)->struct))\
     ((from) ? "GET_STRUCT: %s->%s == nil\n" : "GET_STRUCT: %s == nil%s\n",\
      #from, (from) ? #struct : "")

/*--------------------------------------------------------------------------*/
/* functions for handling parameters                                        */
/*--------------------------------------------------------------------------*/

void add_parameter(int p, const char *key, const char *par);
void Add_parameter(int p, const char *key, const char *par,
		   const char *fname, const char *file, int line);
#define ADD_PARAMETER(p,key,param)\
   Add_parameter(p,key,param,funcName, __FILE__, __LINE__)

void init_parameters(int p, const char *filename);
void init_parameters_cpp(int p, const char *fn, const char *flags);

int get_parameter(int flag, const char *key, const char *format, ...);
int init_param_func_name(const char *, const char *, int call_line);

#define GET_PARAMETER  !init_param_func_name(funcName, __FILE__, __LINE__)?\
                       0 : get_parameter
void save_parameters(const char *file, int info);
void print_parameters(void);

/*--------------------------------------------------------------------------*/
/*  generating filenames, general lists                                     */
/*--------------------------------------------------------------------------*/

const char *generate_filename(const char *path, const char *fn, int ntime);
VOID_LIST_ELEMENT *get_void_list_element(void);
void free_void_list_element(VOID_LIST_ELEMENT *list);

#ifdef __cplusplus
}
#endif

#endif  /* !_ALBERTA_UTIL_H_ */
