/*--------------------------------------------------------------------------*/
/*  PLOT_ANSI: package for simple 2d X11 graphics                           */
/*                                                                          */
/*  (c) by A. Schmidt (1990-2004)                                           */
/*                                                                          */
/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License as published by */
/*     the Free Software Foundation; either version 2 of the License, or    */
/*     any later version.                                                   */
/*                                                                          */
/*     This program is distributed in the hope that it will be useful,      */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*     GNU General Public License for more details.                         */
/*--------------------------------------------------------------------------*/

/* this is the PLOT.h header file */                       /* 25.07.90 schm */

#ifndef _PLOT_
#define _PLOT_

#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#undef XDIRECT
/*#define XDIRECT*/


#ifdef XDIRECT
/*#include <X11/Xd.h>*/
#include <X11/Xdirect.h>
#else
typedef int WindowHandle;
#endif

/* PLOT_EXTERN should be defined only in PLOT_open.c */

#ifndef PLOT_EXTERN
#define PLOT_EXTERN extern
#define PLOT_EXTERN_FUNCTIONS
#define PLOT_INITIALIZE
#endif


/************************************************************/

PLOT_EXTERN int PLOT_redraw;

/************************************************************/

#define MALLOC(size) (char *)malloc(size)
#define MFREE(ptr,size)  free(ptr)


/* different matrix types (for PLOT_transform calls) */

#define PLOT_MODEL_MATRIX 1
#define PLOT_VIEW_MATRIX 2
#define PLOT_PROJECT_MATRIX 3

/* PLOT log item types */

#define PLOT_LOG_null      0 
#define PLOT_LOG_color     1 
#define PLOT_LOG_move      2
#define PLOT_LOG_draw      3
#define PLOT_LOG_triang    4
#define PLOT_LOG_triangRGB 5
#define PLOT_LOG_text      6
#define PLOT_LOG_string    7
#define PLOT_LOG_icolor    8

/************************************************************/

#define PLOT_LOG_SIZE 1000
#define PLOT_LOG_charlen 12  /* 3 * sizeof(float) */

typedef struct {
  int type;
  union {
    float f[3];
    char  c[PLOT_LOG_charlen];
    int   i[3];
  } U;
} PLOT_LOG_ITEM;

typedef struct PLOT_LOG_struct {
  struct PLOT_LOG_struct *prev, *next;
  int used, flushed;
  PLOT_LOG_ITEM item[PLOT_LOG_SIZE];
} PLOT_LOG;



typedef struct PLOT_INFO_struct{
  struct PLOT_INFO_struct *prev, *next;
  Display *display;
  Window window;
  WindowHandle *wh;
  GC gc;
  int windowWidth, windowHeight;
  PLOT_LOG *log;
  int current_color;
  float background_color[3];
  float line_color[3];
  float patch_color[3];
  float text_color[3];
  /* XdPoint line_points[2]; */
  float model_matrix[3][3];
  float view_matrix[3][3];
  float project_matrix[3][3];
  float W2W_matrix[3][3];
  int icolor;
} PLOT_INFO;


/************************************************************/

#ifdef PLOT_EXTERN_FUNCTIONS

/* extern PLOT_INFO *PLOT_create_window(); */
/* extern void          PLOT_destroy_window(); */
/* extern PLOT_INFO *PLOT_get_std_window(); */
/* extern PLOT_INFO *PLOT_set_std_window(); */
#endif /* PLOT_EXTERN_FUNCTIONS */


/*****   NEU  NEU   NEU                                     ****/

/*****  file  PLOT_open.c                                   ****/
extern void PLOT_ALARM_HANDLER(int);
extern void PLOT_alarm(unsigned sec);
extern PLOT_INFO *PLOT_create_window(const char * name, const char *geometry);
extern void PLOT_destroy_window(PLOT_INFO *info);
extern void PLOT_expose(Window window);
extern PLOT_INFO *PLOT_get_std_window(void);
extern PLOT_INFO *PLOT_set_std_window(PLOT_INFO *new_window);
extern void PLOT_title(char *title);
extern void free_log(PLOT_LOG *log);

/*****  file  PLOTlog.c                                     ****/

extern void PLOT_LOG_flush(PLOT_INFO  *info);
extern PLOT_LOG_ITEM *PLOT_LOG_get_item(void);

/*****  file  PLOT2d.c                                      ****/

void PLOT_set_icolor(int icolor);
void PLOT_get_icolor(int *icolor);
void PLOT_set_backgroundcolor(const float color[3]);
void PLOT_get_backgroundcolor(float color[3]);
void PLOT_set_linecolor(const float color[3]);
void PLOT_get_linecolor(float color[3]);
void PLOT_set_patchcolor(const float color[3]);
void PLOT_get_patchcolor(float color[3]);
void PLOT_set_textcolor(const float color[3]);
void PLOT_get_textcolor(float color[3]);
void PLOT_flush(void);
void PLOT_clear(void);
void PLOT_set_transform2d(int mtype, float mat[3][3]);
void PLOT_get_transform2d(int mtype, float mat[3][3]);
void set_W2W_matrix2d(PLOT_INFO *info);
void PLOT_world2d(float xmin, float xmax, float ymin, float ymax);
void PLOT_move2d(float x, float y);
void PLOT_vmove2d(float vec[2]);
void PLOT_draw2d(float x, float y);
void PLOT_vdraw2d(float vec[2]);
void PLOT_line2d(float x1,float y1,float x2,float y2);
void PLOT_vline2d(float vec1[2], float vec2[2]);
void PLOT_triangle2d(float x1, float y1, float x2, float y2,
		     float x3, float y3);
void PLOT_vtriangle2d(float vec1[2], float vec2[2], float vec3[2]);
void PLOT_triangleRGB2d(float x1, float y1, float x2, float y2,
			float x3, float y3,
			float rgb1[3], float rgb2[3], float rgb3[3]);
void PLOT_vtriangleRGB2d(float vec1[2], float vec2[2], float vec3[2],
			 float rgb1[3], float rgb2[3], float rgb3[3]);
void PLOT_text2d(float x, float y, char *string);
void PLOT_vtext2d(float vec[2], char *string);

#endif  /* _PLOT_ */

