/*--------------------------------------------------------------------------*/
/*  PLOT_ANSI: package for simple 2d X11 graphics                           */
/*                                                                          */
/*  (c) by A. Schmidt (1990-2004)                                           */
/*                                                                          */
/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License as published by */
/*     the Free Software Foundation; either version 2 of the License, or    */
/*     any later version.                                                   */
/*                                                                          */
/*     This program is distributed in the hope that it will be useful,      */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*     GNU General Public License for more details.                         */
/*--------------------------------------------------------------------------*/

#include <stdio.h>

#include <fcntl.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

#include "PLOTansi.h"

static int zeichne(int dmy);

main(int argc, char *argv[])
{

  XEvent report;
  Display *display;
  Window window;
  PLOT_INFO *info1, *info2;
  int WHILE;

  info1 = PLOT_create_window("plot test 1", "500x500");
  if (info1==NULL) {
    exit (1);
  }
  printf("info1 = %d\n", info1);


  info2 = PLOT_create_window("plot test 2", "");
  if (info2==NULL) {
    exit (1);
  }
  printf("info2 = %d\n", info2);


  PLOT_set_std_window(info1);
  zeichne(1);

  PLOT_set_std_window(info2);
  zeichne(2);

  PLOT_alarm(1);


  display = info1->display;


  WHILE = 1;
  while (WHILE) {
    /* sleep(5) ; zeichne(2); */

    /*XNextEvent(display, &report); */

    XMaskEvent(display, KeyPressMask | ButtonPressMask , &report);

    switch (report.type) {
    case Expose:
      window = report.xexpose.window;
      printf("expose event window=%d\n",window); 
      while (XCheckWindowEvent(display, window, 
			       Expose | ConfigureNotify, &report)) ;
      PLOT_expose(window);
      /* zeichne(1); */
      break;
    case ConfigureNotify:
      window = report.xexpose.window;
      printf("configure event window=%d\n",window);
      while (XCheckWindowEvent(display, window, 
			       Expose | ConfigureNotify, &report)) ;
      PLOT_expose(window);
      /* zeichne(2); */
      break;
    case ButtonPress:
      printf(" Pressed Button %d\n", report.xbutton.button);
      if (report.xbutton.button == 1) {
	PLOT_title("new title, from now on");
      }
      if (report.xbutton.button == 2) {
	PLOT_destroy_window(info2);
	info2 = NULL;
      }
      if (report.xbutton.button == 3) WHILE = 0;
      break;
    case KeyPress:
      printf(" Pressed Key %d (0x%X)\n",
	     report.xkey.keycode,report.xkey.keycode);
      PLOT_set_std_window(info1); 
      zeichne(3);  
      break;
    default:
      break;
    }
  }

  PLOT_destroy_window(info1);  
  PLOT_destroy_window(info2);

  exit(0);

}


static float model[3][3] = { { 0.015, 0.000, 0.100},
		 	     { 0.000, 0.015, 0.050},
			     { 0.000, 0.000, 1.000} };



static int zeichne(int dmy)
{
  float drei[9][3];
  float farb[3][3];
  int i,j;

/*
  {
    PLOT_INFO *info;
    info = PLOT_get_std_window();
    printf("Habe Zeichenbefehl bekommen %d fuer window %d \n",dmy, info); 
    fflush(stdout);
  }
*/


/*  PLOT_redraw = 0; */
/*  XdReset(); */
 redraw:

#define REDRAW(N) \
  if (PLOT_redraw) {PLOT_redraw = 0; printf("redraw %d...\n",N); goto redraw;}
	
REDRAW(1)
  farb[0][0] = 0.5; farb[0][1] = 0.0; farb[0][2] = 0.2;
  PLOT_set_backgroundcolor(farb);

  PLOT_clear();
REDRAW(2)

  PLOT_set_transform2d( PLOT_MODEL_MATRIX, model );


  farb[0][0] = 1.0; farb[0][1] = 0.3; farb[0][2] = 0.2;
  drei[0][0] = 00.0; drei[0][1] = 00.0; drei[0][2] = 005.0;
  drei[1][0] = 50.0; drei[1][1] = 00.0; drei[1][2] = 005.0;
  drei[2][0] = 00.0; drei[2][1] = 30.0; drei[2][2] = 005.0;
  farb[1][0] = 0.0; farb[1][1] = 1.0; farb[1][2] = 0.3;
  drei[3][0] = 00.0; drei[3][1] = 50.0; drei[3][2] = 002.0;
  drei[4][0] = 50.0; drei[4][1] = 50.0; drei[4][2] = 002.0;
  drei[5][0] = 00.0; drei[5][1] = 10.0; drei[5][2] = 002.0;
  farb[2][0] = 0.0; farb[2][1] = 0.0; farb[2][2] = 0.5;
  drei[6][0] = 50.0; drei[6][1] = 40.0; drei[6][2] = 003.0;
  drei[7][0] = 50.0; drei[7][1] = 10.0; drei[7][2] = 003.0;
  drei[8][0] = 00.0; drei[8][1] = 25.0; drei[8][2] = 003.0;

  PLOT_set_linecolor(farb+1);

REDRAW(5)
   
  drei[0][0] =  1.0; drei[0][1] =  1.0; drei[0][2] =   1.0;
  drei[1][0] =  1.0; drei[1][1] = 45.0; drei[1][2] =   1.0;
  drei[2][0] = 25.0; drei[2][1] = 45.0; drei[2][2] =   1.0;

  PLOT_vmove2d(drei[0]);
  PLOT_vdraw2d(drei[1]);
REDRAW(6)

  PLOT_vline2d(drei[1], drei[2]);
REDRAW(7)


  PLOT_set_patchcolor(farb+1);
  PLOT_vtriangle2d(drei[3],drei[4],drei[5]);

  PLOT_set_patchcolor(farb);
  PLOT_vtriangle2d(drei[0],drei[1],drei[2]);

  PLOT_set_patchcolor(farb+2);
  PLOT_vtriangle2d(drei[6],drei[7],drei[8]);


  if (dmy < 2) {
    drei[0][0] = 20.0; drei[0][1] = 40.0; drei[0][2] =   45.0;
    drei[1][0] =  5.0; drei[1][1] = 15.0; drei[1][2] =   45.0;
    drei[2][0] = 35.0; drei[2][1] =  5.0; drei[2][2] =   45.0;
    farb[0][0] = 1.0; farb[0][1] = 0.0; farb[0][2] = 0.0;
    farb[1][0] = 0.0; farb[1][1] = 0.0; farb[1][2] = 1.0;
    farb[2][0] = 0.0; farb[2][1] = 1.0; farb[2][2] = 0.0;

    PLOT_vtriangleRGB2d(drei[0],drei[1],drei[2], farb[0],farb[1],farb[2]);
  }

  farb[0][0] = 1.0; farb[0][1] = 0.8; farb[0][2] = 0.3;
  PLOT_set_linecolor(farb);

  for (i=1;i<11;i++) {
    drei[0][0] =   1.0;
    drei[0][1] = i*1.0;
    drei[0][2] =   1.0;
    
    for (j=1;j<11;j++) {
      drei[1][0] =  11.0;
      drei[1][1] = j*1.0;
      drei[1][2] =   2.0;

      PLOT_vline2d(drei[0], drei[1]);
    }
  }

REDRAW(8)

  PLOT_flush();
	
REDRAW(9)

#undef REDRAW	
  return(1);
}

